using System;

using CMS.GlobalHelper;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.FormControls;

public partial class CMSModules_Messaging_FormControls_MessageUserSelector : FormEngineUserControl
{
    #region "Private variables"

    private bool? mVisisble = null;

    #endregion


    #region "Public properties"

    /// <summary>
    /// Gets or sets the enabled state of the control
    /// </summary>
    public override bool Enabled
    {
        get
        {
            return base.Enabled;
        }
        set
        {
            base.Enabled = value;
            txtUser.Enabled = value;
            btnSelect.Enabled = value;
        }
    }


    /// <summary>
    /// Name of the selected user
    /// </summary>
    public string SelectedUserName
    {
        get
        {
            UserInfo ui = GetUser();
            return ui != null ? ui.UserName : string.Empty;
        }
        set
        {
            UserInfo ui = UserInfoProvider.GetUserInfo(value);
            if (ui != null)
            {
                txtUser.Text = ui.UserName;
                hiddenField.Value = ui.UserName;
            }
        }
    }


    /// <summary>
    /// ID of the selected user
    /// </summary>
    public int SelectedUserID
    {
        get
        {
            UserInfo ui = GetUser();
            return ui != null ? ui.UserID : 0;
        }
        set
        {
            UserInfo ui = UserInfoProvider.GetUserInfo(value);
            if (ui != null)
            {
                txtUser.Text = ui.UserName;
                hiddenField.Value = ui.UserName;
            }
            else
            {
                txtUser.Text = string.Empty;
                hiddenField.Value = string.Empty;
            }
        }
    }


    /// <summary>
    /// Visibility of control
    /// </summary>
    public override bool Visible
    {
        get
        {
            if (mVisisble.HasValue)
            {
                return mVisisble.Value;
            }
            else
            {
                return base.Visible;
            }
        }
        set
        {
            mVisisble = value;
            base.Visible = value;
        }
    }

    #endregion


    #region "Private methods"

    /// <summary>
    /// Returns UserInfo object from hidden field value (because readonly textbox)
    /// </summary>
    private UserInfo GetUser()
    {
        return UserInfoProvider.GetUserInfo(hiddenField.Value);
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            // Do nothing
        }
        else
        {
            btnSelect.Text = ResHelper.GetString("General.Select");

            ScriptHelper.RegisterClientScriptBlock(this, typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY,
                                                   ScriptHelper.DialogScript);

            ScriptHelper.RegisterClientScriptBlock(this, typeof(string), "fillScript", ScriptHelper.GetScript("function FillUserName(mText, mId, mId2) {document.getElementById(mId).value = mText;document.getElementById(mId2).value = mText;}"));

            string showTab = CMSContext.CurrentUser.IsPublic() ? "Search" : "ContactList";
            string url = "~/CMSModules/Messaging/Dialogs/MessageUserSelector_Frameset.aspx";
            if (IsLiveSite)
            {
                if (!CMSContext.CurrentUser.IsPublic())
                {
                    url = "~/CMSModules/Messaging/CMSPages/MessageUserSelector_Frameset.aspx";
                }
                else
                {
                    url = "~/CMSModules/Messaging/CMSPages/PublicMessageUserSelector.aspx";
                }
            }

            btnSelect.OnClientClick = "modalDialog('" + ResolveUrl(url) +
                                      "?showbuttons=both&refresh=false&showaslink=true&showtab=" + showTab + "&hidid=" +
                                      hiddenField.ClientID + "&mid=" + txtUser.ClientID +
                                      "','MessageUserSelector',600, 510); return false;";

            txtUser.Text = SelectedUserName;
        }
    }

    #endregion
}
